/*
    This file is part of AI Library - Tester
    Copyright (C) 2009-2010  OpenTTD NoAI Community

    AI Library - Tester is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    AI Library - Tester is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AI Library - Tester; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

// Author      : fanioz
// Create Date : 11/1/2009 8:35:18 AM

import("AILib.Tile", "CLTile", 1);
import("AILib.Direction", "CLDir", 1);
import("AILib.Common", "CLCommon", 2);
import("AILib.List", "CLList", 2);
import("AILib.String", "CLString", 2);

class FNStatic
{
    static function BuildSignTown(town_id)
    {
        local id = AISign.BuildSign(AITown.GetLocation(town_id), "X");
        AILog.Info("X-marked on town :" + AITown.GetName(town_id));
        AIController.Sleep(10);
        AISign.RemoveSign(id);
        return 1;
    }
}

class Test extends AIController
{
    Title = null;
    Item = null;
    constructor()
    {
        Title = ["[*]", "AI for Common Library Tester", "[*]"];
        Item = ["->", "", "<-"];
    }
    function Start()
    {
        Sleep(5);
        local line = CLString.Repeat("-", 50);
        AILog.Info(line);
        PrintTitle();
        AILog.Info(line);

        Test_Common();
        Test_List();
        Test_String();
        Test_Direction();
    }

    function PrintTitle()
    {
        AILog.Info("");
        AILog.Warning(CLString.Join(Title, " "));
    }

    function PrintItem(text)
    {
        AILog.Info("");
        Item[1] = text;
        AILog.Info(CLString.Join(Item, " "));
    }

    function Test_Common()
    {
        Title[1] = "Lib.Common Test";
        PrintTitle();

        local ver = CLCommon.GetVersion();
        PrintItem("CLCommon.GetVersion()");

        local ver = CLCommon.GetVersion();
        AILog.Info("Run On OpenTTD Ver: " + ver.Major + "." + ver.Minor +
                   " Build:" + ver.Build + "(" +
                   (ver.IsRelease ?  " Release" : " Rev." + ver.Revision) + " )");

        PrintItem("CLCommon.ACall()");
        //instance call
        CLCommon.ACall(Test_ACall, [this, 1, 2, 3, 4, 5, 6, "instance call"]);
        //static call
        CLCommon.ACall(Test.Test_ACall, [null, 1, 2, 3, 4, 5, 6, "static call"]);
    }

    function Test_ACall(a, b, c , d, e, f, g)
    {
        AILog.Info("I believe, I've must been called with ACall");
        AILog.Info("The arguments are:");
        local args = CLString.Join([a, b, c, d, e, f, g], " - ");
        AILog.Info(args);
    }

    function Test_List()
    {
        Title[1] = "Lib.List Test";
        PrintTitle();

        PrintItem("Creating List from AITownList");
        local my_towns = CLList(AITownList());

        PrintItem("Try using Begin(), HasNext(), Next()");

        for(local i = my_towns.Begin(); my_towns.HasNext(); i = my_towns.Next())
        {
            AILog.Info(AITown.GetName(i));
        }

        PrintItem("Try using Begin(), IsEnd(), Next()");

        for(local i = my_towns.Begin(); !my_towns.IsEnd(); i = my_towns.Next())
        {
            AILog.Info(AITown.GetName(i));
        }

        PrintItem("Valuate Do Command in List");
        my_towns.DoValuate(FNStatic.BuildSignTown);

        PrintItem("Standard Valuate (Non-Do Command) in List");
        my_towns.Valuate(AITown.GetLocation);
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Set values of List from their items");
        my_towns.SetValueAsItem();
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Get all of items as an array");
        local tmp = my_towns.GetItemArray();
        AILog.Info("type of tmp= " + typeof tmp + " has lenght: " + tmp.len());

        PrintItem("Get all of values as an array");
        my_towns.Valuate(AITown.GetPopulation);
        tmp = my_towns.GetValueArray();
        AILog.Info("type of tmp= " + typeof tmp + " has lenght: " + tmp.len());

        PrintItem("Add items of AIList from array");
        my_towns.AddItems(tmp);
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Get all of data as an array with [item, value]-pairs");
        tmp = my_towns.GetItemValue();
        foreach(idx in tmp)
        {
            AILog.Info("item:" + idx[0] + " had value: " + idx[1]);
        }
        PrintItem("Add all [item, value]-pairs from a given array to this list");
        my_towns.Clear();
        my_towns.AddItemValue(tmp);
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Sort Item Ascending");
        my_towns.SortItemAscending();
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Sort Item Descending");
        my_towns.SortItemDescending();
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Sort Value Ascending");
        my_towns.SortValueAscending();
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Sort Value Descending");
        my_towns.SortValueDescending();
        AILog.Info(CLString.AIList(my_towns));

        local new_tile = AIMap.GetTileIndex(10, 10);
        PrintItem("Adding a tile into list");
        my_towns.AddTile(new_tile);
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Removing a tile from list");
        my_towns.RemoveTile(new_tile);
        AILog.Info(CLString.AIList(my_towns));

        local second_tile = AIMap.GetTileIndex(13, 13);
        PrintItem("Adds the rectangle between t1 and t2 to the list");
        my_towns.AddRectangle(new_tile, second_tile);
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Remove the rectangle between t1 and t2 from the list");
        my_towns.RemoveRectangle(new_tile, second_tile);
        AILog.Info(CLString.AIList(my_towns));

        PrintItem("Pop from list");
        AILog.Info("Top item was :" + my_towns.Pop());

        PrintItem("Peek from list");
        AILog.Info("Top item now :" + my_towns.Peek());

        PrintItem("Push '9999' into list");
        my_towns.Push(9999);
        AILog.Info(CLString.AIList(my_towns));

        AILog.Info("");
        AILog.Info("item count:" + my_towns.Count());

        PrintItem("CountIfRemoveBetweenValue(-1, 100)");
        AILog.Info("item left:" + my_towns.CountIfRemoveBetweenValue(-1, 100));

        PrintItem("CountIfRemoveAboveValue(100)");
        AILog.Info("item left:" + my_towns.CountIfRemoveAboveValue(100));

        PrintItem("CountIfRemoveBelowValue(10)");
        AILog.Info("item left:" + my_towns.CountIfRemoveBelowValue(10));

        PrintItem("CountIfRemoveValue(0)");
        AILog.Info("item left:" + my_towns.CountIfRemoveValue(0));

        PrintItem("CountIfKeepValue(0)");
        AILog.Info("item left:" + my_towns.CountIfKeepValue(0));

        PrintItem("CountIfRemoveTop(2)");
        AILog.Info("item left:" + my_towns.CountIfRemoveTop(2));

        PrintItem("CountIfRemoveBottom(3)");
        AILog.Info("item left:" + my_towns.CountIfRemoveBottom(3));
    }

    function Test_String()
    {
        // at least you've seen how .Join and .Repeat was used

        Title[1] = "Lib.String Test";
        PrintTitle();

        local today = AIDate.GetCurrentDate();
        PrintItem("Date in DMY");
        AILog.Info(CLString.Date(today, CLString.DateDMY, "-"));

        PrintItem("Date in MDY");
        AILog.Info(CLString.Date(today, CLString.DateMDY, ","));

        PrintItem("Date in YMD");
        AILog.Info(CLString.Date(today, CLString.DateYMD, "."));

        local txt = "this-string-should-be-splitted";
        AILog.Info("Original text was : " + txt);
        PrintItem("Split");
        local my_array = CLString.Split(txt, "-");
        AILog.Info("splitted into =" + my_array.len());
        foreach(str in my_array)
        {
            AILog.Info("index:" + " -> " + str);
        }

        PrintItem("Join with *");
        txt = CLString.Join(my_array, "*");
        AILog.Info("Joined text is:" + txt);

        local my_vehicles = AIVehicleList();
        local my_planes = CLList(my_vehicles);
        PrintItem("Vehicle - Airplane - State");
        local cur_vhc_type = AIVehicle.VT_AIR;
        local cur_vhc_txt = CLString.VehicleType(cur_vhc_type);
        my_planes.Valuate(AIVehicle.GetVehicleType);
        my_planes.KeepValue(cur_vhc_type);
        my_planes.Valuate(AIVehicle.GetEngineType);
        foreach(vhc, eng in my_planes)
        {
            AILog.Info(cur_vhc_txt + " no:" + AIVehicle.GetUnitNumber(vhc) + " is " + CLString.PlaneType(AIEngine.GetPlaneType(eng)));
            AILog.Info(CLString.VehicleState(AIVehicle.GetState(vhc)));
        }

        local my_roads = CLList(my_vehicles);
        PrintItem("Vehicle - Road - State");
        cur_vhc_type = AIVehicle.VT_ROAD;
        cur_vhc_txt = CLString.VehicleType(cur_vhc_type);
        my_roads.Valuate(AIVehicle.GetVehicleType);
        my_roads.KeepValue(cur_vhc_type);
        my_roads.Valuate(AIVehicle.GetRoadType);
        foreach(vhc, rtt in my_roads)
        {
            AILog.Info(cur_vhc_txt + " no:" + AIVehicle.GetUnitNumber(vhc) + " is " + CLString.RoadTrackType(rtt));
            AILog.Info(CLString.VehicleState(AIVehicle.GetState(vhc)));
        }

        local my_rails = CLList(my_vehicles);
        PrintItem("Vehicle - Rail - State");
        cur_vhc_type = AIVehicle.VT_RAIL;
        cur_vhc_txt = CLString.VehicleType(cur_vhc_type);
        my_rails.Valuate(AIVehicle.GetVehicleType);
        my_rails.KeepValue(cur_vhc_type);
        my_rails.Valuate(AIVehicle.GetEngineType);
        foreach(vhc, eng in my_rails)
        {
            AILog.Info(cur_vhc_txt + " no:" + AIVehicle.GetUnitNumber(vhc) + " is " + CLString.RailTrackType(AIEngine.GetRailType(eng)));
            AILog.Info(CLString.VehicleState(AIVehicle.GetState(vhc)));
        }

        PrintItem("Vehicle - Water - State");
        cur_vhc_type = AIVehicle.VT_WATER;
        cur_vhc_txt = CLString.VehicleType(cur_vhc_type);
        my_vehicles.Valuate(AIVehicle.GetVehicleType);
        my_vehicles.KeepValue(cur_vhc_type);
        foreach(vhc, _ in my_vehicles)
        {
            AILog.Info(cur_vhc_txt + " no:" + AIVehicle.GetUnitNumber(vhc));
            AILog.Info(CLString.VehicleState(AIVehicle.GetState(vhc)));
        }

        PrintItem("Tile & Station & Airport Type");
        local all_st = [AIStation.STATION_TRAIN, AIStation.STATION_TRUCK_STOP, AIStation.STATION_BUS_STOP, AIStation.STATION_AIRPORT, AIStation.STATION_DOCK];
        foreach(sta in all_st)
        {
            local my_station = AIStationList(sta);
            my_station.Valuate(AIStation.GetLocation);
            foreach(id, loc in my_station)
            {
                assert(AIStation.HasStationType(id, sta));
                AILog.Info("station named " + AIStation.GetName(id) + " was " + CLString.StationType(sta));
                AILog.Info("located at: " + CLString.Tile(loc));

                if(sta == AIStation.STATION_AIRPORT)
                {
                    AILog.Info("Airport type : " + CLString.AirportType(AIAirport.GetAirportType(loc)));
                }

                AILog.Info("");
            }
        }

        PrintItem("Rail direction");
        local tiles = CLList();
        tiles.AddTile(AIMap.GetTileIndex(32, 56));
        tiles.AddTile(AIMap.GetTileIndex(33, 56));
        tiles.AddTile(AIMap.GetTileIndex(33, 55));
        tiles.AddTile(AIMap.GetTileIndex(32, 55));
        tiles.AddTile(AIMap.GetTileIndex(32, 54));
        tiles.AddTile(AIMap.GetTileIndex(32, 53));

        AIRail.SetCurrentRailType(AIRail.GetRailType(tiles.Peek()));

        while(!tiles.IsEmpty())
        {
            local tile = tiles.Pop();
            AILog.Info("Rail dir on: " + CLString.Tile(tile) + " is " + CLString.RailDirection(AIRail.GetRailTracks(tile)));
        }

        PrintItem("Town effect");
        foreach(cargo, _ in AICargoList())
        {
            AILog.Info("Cargo :" + AICargo.GetCargoLabel(cargo) + " ," + CLString.TownEffect(AICargo.GetTownEffect(cargo)));
        }

        PrintItem("Table");
        local new_table =
        {
            _name = "String"
            _type = "AI Library"
            _version = "1.0"
        };
        AILog.Info(CLString.Table(new_table));
    }

    function Test_Direction()
    {
        Title[1] = "Lib.Direction Test";
        PrintTitle();

        PrintItem("All direction")
        local dir_list = CLDir.GetAllDirsInRandomOrder();
        foreach(dir, _ in dir_list)
        {
            AILog.Info("Direction :" + CLDir.GetDirString(dir));
        }

        PrintItem("Main direction")
        dir_list = CLDir.GetMainDirsInRandomOrder();
        foreach(dir, _ in dir_list)
        {
            AILog.Info("Direction :" + CLDir.GetDirString(dir));
        }

        PrintItem("Diagonal direction")
        dir_list = CLDir.GetDiagonalDirsInRandomOrder();
        foreach(dir, _ in dir_list)
        {
            AILog.Info("Direction :" + CLDir.GetDirString(dir));
        }

        local cur_tile = AIMap.GetTileIndex(41, 45);
        local next_tile = AIMap.GetTileIndex(40, 45);

        PrintItem("GetDirectionToAdjacentTile");
        local dir_str = CLDir.GetDirString(CLDir.GetDirectionToAdjacentTile(cur_tile, next_tile));
        AISign.BuildSign(cur_tile, "x");
        AISign.BuildSign(next_tile, dir_str);
        AILog.Info("Direction from X was" + dir_str);

        next_tile = AIMap.GetTileIndex(41, 54);
        PrintItem("GetDirectionToTile");
        dir_str = CLDir.GetDirString(CLDir.GetDirectionToTile(cur_tile, next_tile));
        AISign.BuildSign(next_tile, dir_str);
        AILog.Info("Direction from X was" + dir_str);

        PrintItem("Turning Direction");
        PrintItem("TurnDirClockwise45Deg");
        local cur_dir = CLDir.DIR_FIRST;
        local res_dir = CLDir.Turn(cur_dir, CLDir.TURN_45, CLDir.CLOCKWISE, 1);
        dir_str = CLDir.GetDirString(cur_dir);

        AILog.Info("Start direction was " + dir_str);
        AILog.Info("Direction after turn clockwise 45 deg once= " + CLDir.GetDirString(res_dir));

        PrintItem("TurnDirAntiClockwise45Deg");
        res_dir = CLDir.Turn(cur_dir, CLDir.TURN_45, CLDir.ANTI_CLOCKWISE, 1);
        AILog.Info("Start direction was " + dir_str);
        AILog.Info("Direction after turn anti-clockwise 45 deg once= " + CLDir.GetDirString(res_dir));

        PrintItem("TurnDirClockwise90Deg");
        res_dir = CLDir.Turn(cur_dir, CLDir.TURN_90, CLDir.CLOCKWISE, 1);
        AILog.Info("Start direction was " + dir_str);
        AILog.Info("Direction after turn clockwise 90 deg once= " + CLDir.GetDirString(res_dir));

        PrintItem("TurnDirAntiClockwise90Deg");
        res_dir = CLDir.Turn(cur_dir, CLDir.TURN_90, CLDir.ANTI_CLOCKWISE, 1);
        AILog.Info("Start direction was " + dir_str);
        AILog.Info("Direction after turn anti-clockwise 90 deg once= " + CLDir.GetDirString(res_dir));

        PrintItem("OppositeDir");
        res_dir = CLDir.Turn(cur_dir, CLDir.TURN_90, CLDir.CLOCKWISE, 2);
        AILog.Info("Start direction was " + dir_str);
        AILog.Info("Direction after turn opposite= " + CLDir.GetDirString(res_dir));

    }
}
